@@ -65,12 +65,14 @@ enum
   ARG_YPAD,
   ARG_DELTAX,
   ARG_DELTAY,
+  ARG_SILENT,
   ARG_FONT_DESC
 };
 
 #define DEFAULT_YPAD 25
 #define DEFAULT_XPAD 25
 #define DEFAULT_FONT "sans"
+#define DEFAULT_SILENT FALSE
 
 #define GST_CAIRO_TEXT_OVERLAY_DEFAULT_SCALE   20.0
 
@@ -108,6 +110,8 @@ static GstFlowReturn gst_text_overlay_collected (GstCollectPads * pads,
     gpointer data);
 static void gst_text_overlay_finalize (GObject * object);
 static void gst_text_overlay_font_init (GstCairoTextOverlay * overlay);
+static gboolean gst_text_overlay_src_event (GstPad * pad, GstEvent * event);
+static gboolean gst_text_overlay_video_event (GstPad * pad, GstEvent * event);
 
 /* These macros are adapted from videotestsrc.c */
 #define I420_Y_ROWSTRIDE(width) (GST_ROUND_UP_4(width))
@@ -128,12 +132,12 @@ gst_text_overlay_base_init (gpointer g_class)
 {
   GstElementClass *element_class = GST_ELEMENT_CLASS (g_class);
 
-  gst_element_class_add_pad_template (element_class,
-      gst_static_pad_template_get (&cairo_text_overlay_src_template_factory));
-  gst_element_class_add_pad_template (element_class,
-      gst_static_pad_template_get (&video_sink_template_factory));
-  gst_element_class_add_pad_template (element_class,
-      gst_static_pad_template_get (&text_sink_template_factory));
+  gst_element_class_add_static_pad_template (element_class,
+      &cairo_text_overlay_src_template_factory);
+  gst_element_class_add_static_pad_template (element_class,
+      &video_sink_template_factory);
+  gst_element_class_add_static_pad_template (element_class,
+      &text_sink_template_factory);
 
   gst_element_class_set_details_simple (element_class, "Text overlay",
       "Filter/Editor/Video",
@@ -199,6 +203,11 @@ gst_text_overlay_class_init (GstCairoTextOverlayClass * klass)
           "See documentation of "
           "pango_font_description_from_string"
           " for syntax.", "", G_PARAM_WRITABLE | G_PARAM_STATIC_STRINGS));
+  /* FIXME 0.11: rename to "visible" or "text-visible" or "render-text" */
+  g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_SILENT,
+      g_param_spec_boolean ("silent", "silent",
+          "Whether to render the text string",
+          DEFAULT_SILENT, G_PARAM_WRITABLE | G_PARAM_STATIC_STRINGS));
 }
 
 static void
@@ -248,6 +257,8 @@ gst_text_overlay_init (GstCairoTextOverlay * overlay,
       (&cairo_text_overlay_src_template_factory, "src");
   gst_pad_set_getcaps_function (overlay->srcpad,
       GST_DEBUG_FUNCPTR (gst_text_overlay_getcaps));
+  gst_pad_set_event_function (overlay->srcpad,
+      GST_DEBUG_FUNCPTR (gst_text_overlay_src_event));
   gst_element_add_pad (GST_ELEMENT (overlay), overlay->srcpad);
 
   overlay->halign = GST_CAIRO_TEXT_OVERLAY_HALIGN_CENTER;
@@ -263,6 +274,8 @@ gst_text_overlay_init (GstCairoTextOverlay * overlay,
   overlay->font = g_strdup (DEFAULT_FONT);
   gst_text_overlay_font_init (overlay);
 
+  overlay->silent = DEFAULT_SILENT;
+
   overlay->fps_n = 0;
   overlay->fps_d = 1;
 
@@ -274,6 +287,14 @@ gst_text_overlay_init (GstCairoTextOverlay * overlay,
   overlay->video_collect_data = gst_collect_pads_add_pad (overlay->collect,
       overlay->video_sinkpad, sizeof (GstCollectData));
 
+  /* FIXME: hacked way to override/extend the event function of
+   * GstCollectPads; because it sets its own event function giving the
+   * element no access to events. Nicked from avimux. */
+  overlay->collect_event =
+      (GstPadEventFunction) GST_PAD_EVENTFUNC (overlay->video_sinkpad);
+  gst_pad_set_event_function (overlay->video_sinkpad,
+      GST_DEBUG_FUNCPTR (gst_text_overlay_video_event));
+
   /* text pad will be added when it is linked */
   overlay->text_collect_data = NULL;
 }
@@ -398,6 +419,9 @@ gst_text_overlay_set_property (GObject * object, guint prop_id,
       gst_text_overlay_font_init (overlay);
       break;
     }
+    case ARG_SILENT:
+      overlay->silent = g_value_get_boolean (value);
+      break;
     default:{
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -419,6 +443,11 @@ gst_text_overlay_render_text (GstCairoTextOverlay * overlay,
   gchar *string;
   double x, y;
 
+  if (overlay->silent) {
+    GST_DEBUG_OBJECT (overlay, "Silent mode, not rendering");
+    return;
+  }
+
   if (textlen < 0)
     textlen = strlen (text);
 
@@ -942,6 +971,45 @@ done:
   }
 }
 
+static gboolean
+gst_text_overlay_src_event (GstPad * pad, GstEvent * event)
+{
+  GstCairoTextOverlay *overlay =
+      GST_CAIRO_TEXT_OVERLAY (gst_pad_get_parent (pad));
+  gboolean ret = TRUE;
+
+  /* forward events to the video sink, and, if it is linked, the text sink */
+  if (overlay->text_collect_data) {
+    gst_event_ref (event);
+    ret &= gst_pad_push_event (overlay->text_sinkpad, event);
+  }
+  ret &= gst_pad_push_event (overlay->video_sinkpad, event);
+
+  gst_object_unref (overlay);
+  return ret;
+}
+
+static gboolean
+gst_text_overlay_video_event (GstPad * pad, GstEvent * event)
+{
+  gboolean ret = FALSE;
+  GstCairoTextOverlay *overlay = NULL;
+
+  overlay = GST_CAIRO_TEXT_OVERLAY (gst_pad_get_parent (pad));
+
+  if (GST_EVENT_TYPE (event) == GST_EVENT_NEWSEGMENT) {
+    GST_DEBUG_OBJECT (overlay,
+        "received new segment on video sink pad, forwarding");
+    gst_event_ref (event);
+    gst_pad_push_event (overlay->srcpad, event);
+  }
+
+  /* now GstCollectPads can take care of the rest, e.g. EOS */
+  ret = overlay->collect_event (pad, event);
+  gst_object_unref (overlay);
+  return ret;
+}
+
 static GstStateChangeReturn
 gst_text_overlay_change_state (GstElement * element, GstStateChange transition)
 {
