@@ -265,10 +265,10 @@ gst_cairo_time_overlay_base_init (gpointer g_class)
       "Filter/Editor/Video",
       "Overlays the time on a video stream", "David Schleef <ds@schleef.org>");
 
-  gst_element_class_add_pad_template (element_class,
-      gst_static_pad_template_get (&gst_cairo_time_overlay_sink_template));
-  gst_element_class_add_pad_template (element_class,
-      gst_static_pad_template_get (&gst_cairo_time_overlay_src_template));
+  gst_element_class_add_static_pad_template (element_class,
+      &gst_cairo_time_overlay_sink_template);
+  gst_element_class_add_static_pad_template (element_class,
+      &gst_cairo_time_overlay_src_template);
 }
 
 static void
