@@ -166,10 +166,8 @@ gst_dvdec_base_init (gpointer g_class)
 {
   GstElementClass *element_class = GST_ELEMENT_CLASS (g_class);
 
-  gst_element_class_add_pad_template (element_class,
-      gst_static_pad_template_get (&sink_temp));
-  gst_element_class_add_pad_template (element_class,
-      gst_static_pad_template_get (&src_temp));
+  gst_element_class_add_static_pad_template (element_class, &sink_temp);
+  gst_element_class_add_static_pad_template (element_class, &src_temp);
 
   gst_element_class_set_details_simple (element_class, "DV video decoder",
       "Codec/Decoder/Video",
