@@ -69,6 +69,10 @@ gst_deinterlace_method_supported_impl (GstDeinterlaceMethodClass * klass,
       return (klass->deinterlace_frame_y41b != NULL);
     case GST_VIDEO_FORMAT_AYUV:
       return (klass->deinterlace_frame_ayuv != NULL);
+    case GST_VIDEO_FORMAT_NV12:
+      return (klass->deinterlace_frame_nv12 != NULL);
+    case GST_VIDEO_FORMAT_NV21:
+      return (klass->deinterlace_frame_nv21 != NULL);
     case GST_VIDEO_FORMAT_ARGB:
     case GST_VIDEO_FORMAT_xRGB:
       return (klass->deinterlace_frame_argb != NULL);
@@ -152,6 +156,12 @@ gst_deinterlace_method_setup_impl (GstDeinterlaceMethod * self,
     case GST_VIDEO_FORMAT_AYUV:
       self->deinterlace_frame = klass->deinterlace_frame_ayuv;
       break;
+    case GST_VIDEO_FORMAT_NV12:
+      self->deinterlace_frame = klass->deinterlace_frame_nv12;
+      break;
+    case GST_VIDEO_FORMAT_NV21:
+      self->deinterlace_frame = klass->deinterlace_frame_nv21;
+      break;
     case GST_VIDEO_FORMAT_ARGB:
     case GST_VIDEO_FORMAT_xRGB:
       self->deinterlace_frame = klass->deinterlace_frame_argb;
@@ -196,10 +206,10 @@ gst_deinterlace_method_init (GstDeinterlaceMethod * self)
 void
 gst_deinterlace_method_deinterlace_frame (GstDeinterlaceMethod * self,
     const GstDeinterlaceField * history, guint history_count,
-    GstBuffer * outbuf)
+    GstBuffer * outbuf, int cur_field_idx)
 {
   g_assert (self->deinterlace_frame != NULL);
-  self->deinterlace_frame (self, history, history_count, outbuf);
+  self->deinterlace_frame (self, history, history_count, outbuf, cur_field_idx);
 }
 
 gint
@@ -268,6 +278,12 @@ gst_deinterlace_simple_method_supported (GstDeinterlaceMethodClass * mklass,
     case GST_VIDEO_FORMAT_AYUV:
       return (klass->interpolate_scanline_ayuv != NULL
           && klass->copy_scanline_ayuv != NULL);
+    case GST_VIDEO_FORMAT_NV12:
+      return (klass->interpolate_scanline_nv12 != NULL
+          && klass->copy_scanline_nv12 != NULL);
+    case GST_VIDEO_FORMAT_NV21:
+      return (klass->interpolate_scanline_nv21 != NULL
+          && klass->copy_scanline_nv21 != NULL);
     case GST_VIDEO_FORMAT_I420:
     case GST_VIDEO_FORMAT_YV12:
     case GST_VIDEO_FORMAT_Y444:
@@ -302,14 +318,13 @@ gst_deinterlace_simple_method_copy_scanline_packed (GstDeinterlaceSimpleMethod *
 static void
 gst_deinterlace_simple_method_deinterlace_frame_packed (GstDeinterlaceMethod *
     method, const GstDeinterlaceField * history, guint history_count,
-    GstBuffer * outbuf)
+    GstBuffer * outbuf, gint cur_field_idx)
 {
   GstDeinterlaceSimpleMethod *self = GST_DEINTERLACE_SIMPLE_METHOD (method);
   GstDeinterlaceMethodClass *dm_class = GST_DEINTERLACE_METHOD_GET_CLASS (self);
   GstDeinterlaceScanlineData scanlines;
   guint8 *dest;
-  const guint8 *field0, *field1, *field2, *field3;
-  gint cur_field_idx = history_count - dm_class->fields_required;
+  const guint8 *field0, *field1, *field2, *fieldp;
   guint cur_field_flags = history[cur_field_idx].flags;
   gint i;
   gint frame_height = self->parent.frame_height;
@@ -318,29 +333,29 @@ gst_deinterlace_simple_method_deinterlace_frame_packed (GstDeinterlaceMethod *
   g_assert (self->interpolate_scanline_packed != NULL);
   g_assert (self->copy_scanline_packed != NULL);
 
+  if (cur_field_idx > 0) {
+    fieldp = GST_BUFFER_DATA (history[cur_field_idx - 1].buf);
+  } else {
+    fieldp = NULL;
+  }
+
   dest = GST_BUFFER_DATA (outbuf);
   field0 = GST_BUFFER_DATA (history[cur_field_idx].buf);
 
   g_assert (dm_class->fields_required <= 4);
 
-  if (dm_class->fields_required >= 2) {
+  if (cur_field_idx + 1 < history_count) {
     field1 = GST_BUFFER_DATA (history[cur_field_idx + 1].buf);
   } else {
     field1 = NULL;
   }
 
-  if (dm_class->fields_required >= 3) {
+  if (cur_field_idx + 2 < history_count) {
     field2 = GST_BUFFER_DATA (history[cur_field_idx + 2].buf);
   } else {
     field2 = NULL;
   }
 
-  if (dm_class->fields_required >= 4) {
-    field3 = GST_BUFFER_DATA (history[cur_field_idx + 3].buf);
-  } else {
-    field3 = NULL;
-  }
-
 #define CLAMP_LOW(i) (((i)<0) ? (i+2) : (i))
 #define CLAMP_HI(i) (((i)>=(frame_height)) ? (i-2) : (i))
 #define LINE(x,i) ((x) + CLAMP_HI(CLAMP_LOW(i)) * (stride))
@@ -350,8 +365,11 @@ gst_deinterlace_simple_method_deinterlace_frame_packed (GstDeinterlaceMethod *
     memset (&scanlines, 0, sizeof (scanlines));
     scanlines.bottom_field = (cur_field_flags == PICTURE_INTERLACED_BOTTOM);
 
-    if (!((i & 1) ^ (cur_field_flags == PICTURE_INTERLACED_BOTTOM))) {
+    if (!((i & 1) ^ scanlines.bottom_field)) {
       /* copying */
+      scanlines.tp = LINE2 (fieldp, i - 1);
+      scanlines.bp = LINE2 (fieldp, i + 1);
+
       scanlines.tt0 = LINE2 (field0, (i - 2 >= 0) ? i - 2 : i);
       scanlines.m0 = LINE2 (field0, i);
       scanlines.bb0 = LINE2 (field0, (i + 2 < frame_height ? i + 2 : i));
@@ -363,12 +381,13 @@ gst_deinterlace_simple_method_deinterlace_frame_packed (GstDeinterlaceMethod *
       scanlines.m2 = LINE2 (field2, i);
       scanlines.bb2 = LINE2 (field2, (i + 2 < frame_height ? i + 2 : i));
 
-      scanlines.t3 = LINE2 (field3, i - 1);
-      scanlines.b3 = LINE2 (field3, i + 1);
-
       self->copy_scanline_packed (self, LINE (dest, i), &scanlines);
     } else {
       /* interpolating */
+      scanlines.ttp = LINE2 (fieldp, (i - 2 >= 0) ? i - 2 : i);
+      scanlines.mp = LINE2 (fieldp, i);
+      scanlines.bbp = LINE2 (fieldp, (i + 2 < frame_height ? i + 2 : i));
+
       scanlines.t0 = LINE2 (field0, i - 1);
       scanlines.b0 = LINE2 (field0, i + 1);
 
@@ -379,10 +398,6 @@ gst_deinterlace_simple_method_deinterlace_frame_packed (GstDeinterlaceMethod *
       scanlines.t2 = LINE2 (field2, i - 1);
       scanlines.b2 = LINE2 (field2, i + 1);
 
-      scanlines.tt3 = LINE2 (field3, (i - 2 >= 0) ? i - 2 : i);
-      scanlines.m3 = LINE2 (field3, i);
-      scanlines.bb3 = LINE2 (field3, (i + 2 < frame_height ? i + 2 : i));
-
       self->interpolate_scanline_packed (self, LINE (dest, i), &scanlines);
     }
   }
@@ -436,7 +451,7 @@ gst_deinterlace_simple_method_copy_scanline_planar_v (GstDeinterlaceSimpleMethod
 static void
     gst_deinterlace_simple_method_deinterlace_frame_planar_plane
     (GstDeinterlaceSimpleMethod * self, guint8 * dest, const guint8 * field0,
-    const guint8 * field1, const guint8 * field2, const guint8 * field3,
+    const guint8 * field1, const guint8 * field2, const guint8 * fieldp,
     guint cur_field_flags,
     gint plane, GstDeinterlaceSimpleMethodFunction copy_scanline,
     GstDeinterlaceSimpleMethodFunction interpolate_scanline)
@@ -453,8 +468,11 @@ static void
     memset (&scanlines, 0, sizeof (scanlines));
     scanlines.bottom_field = (cur_field_flags == PICTURE_INTERLACED_BOTTOM);
 
-    if (!((i & 1) ^ (cur_field_flags == PICTURE_INTERLACED_BOTTOM))) {
+    if (!((i & 1) ^ scanlines.bottom_field)) {
       /* copying */
+      scanlines.tp = LINE2 (fieldp, i - 1);
+      scanlines.bp = LINE2 (fieldp, i + 1);
+
       scanlines.tt0 = LINE2 (field0, (i - 2 >= 0) ? i - 2 : i);
       scanlines.m0 = LINE2 (field0, i);
       scanlines.bb0 = LINE2 (field0, (i + 2 < frame_height ? i + 2 : i));
@@ -466,12 +484,13 @@ static void
       scanlines.m2 = LINE2 (field2, i);
       scanlines.bb2 = LINE2 (field2, (i + 2 < frame_height ? i + 2 : i));
 
-      scanlines.t3 = LINE2 (field3, i - 1);
-      scanlines.b3 = LINE2 (field3, i + 1);
-
       copy_scanline (self, LINE (dest, i), &scanlines);
     } else {
       /* interpolating */
+      scanlines.ttp = LINE2 (fieldp, (i - 2 >= 0) ? i - 2 : i);
+      scanlines.mp = LINE2 (fieldp, i);
+      scanlines.bbp = LINE2 (fieldp, (i + 2 < frame_height ? i + 2 : i));
+
       scanlines.t0 = LINE2 (field0, i - 1);
       scanlines.b0 = LINE2 (field0, i + 1);
 
@@ -482,10 +501,6 @@ static void
       scanlines.t2 = LINE2 (field2, i - 1);
       scanlines.b2 = LINE2 (field2, i + 1);
 
-      scanlines.tt3 = LINE2 (field3, (i - 2 >= 0) ? i - 2 : i);
-      scanlines.m3 = LINE2 (field3, i);
-      scanlines.bb3 = LINE2 (field3, (i + 2 < frame_height ? i + 2 : i));
-
       interpolate_scanline (self, LINE (dest, i), &scanlines);
     }
   }
@@ -494,13 +509,12 @@ static void
 static void
 gst_deinterlace_simple_method_deinterlace_frame_planar (GstDeinterlaceMethod *
     method, const GstDeinterlaceField * history, guint history_count,
-    GstBuffer * outbuf)
+    GstBuffer * outbuf, gint cur_field_idx)
 {
   GstDeinterlaceSimpleMethod *self = GST_DEINTERLACE_SIMPLE_METHOD (method);
   GstDeinterlaceMethodClass *dm_class = GST_DEINTERLACE_METHOD_GET_CLASS (self);
   guint8 *out;
-  const guint8 *field0, *field1, *field2, *field3;
-  gint cur_field_idx = history_count - dm_class->fields_required;
+  const guint8 *field0, *field1, *field2, *fieldp;
   guint cur_field_flags = history[cur_field_idx].flags;
   gint i, offset;
   GstDeinterlaceSimpleMethodFunction copy_scanline;
@@ -520,28 +534,73 @@ gst_deinterlace_simple_method_deinterlace_frame_planar (GstDeinterlaceMethod *
 
     out = GST_BUFFER_DATA (outbuf) + offset;
 
+    fieldp = NULL;
+    if (cur_field_idx > 0) {
+      fieldp = GST_BUFFER_DATA (history[cur_field_idx - 1].buf) + offset;
+    }
+
     field0 = GST_BUFFER_DATA (history[cur_field_idx].buf) + offset;
 
     g_assert (dm_class->fields_required <= 4);
 
     field1 = NULL;
-    if (dm_class->fields_required >= 2) {
+    if (cur_field_idx + 1 < history_count) {
       field1 = GST_BUFFER_DATA (history[cur_field_idx + 1].buf) + offset;
     }
 
     field2 = NULL;
-    if (dm_class->fields_required >= 3) {
+    if (cur_field_idx + 2 < history_count) {
       field2 = GST_BUFFER_DATA (history[cur_field_idx + 2].buf) + offset;
     }
 
-    field3 = NULL;
-    if (dm_class->fields_required >= 4) {
-      field3 = GST_BUFFER_DATA (history[cur_field_idx + 3].buf) + offset;
+    gst_deinterlace_simple_method_deinterlace_frame_planar_plane (self, out,
+        field0, field1, field2, fieldp, cur_field_flags, i, copy_scanline,
+        interpolate_scanline);
+  }
+}
+
+static void
+gst_deinterlace_simple_method_deinterlace_frame_nv12 (GstDeinterlaceMethod *
+    method, const GstDeinterlaceField * history, guint history_count,
+    GstBuffer * outbuf, gint cur_field_idx)
+{
+  GstDeinterlaceSimpleMethod *self = GST_DEINTERLACE_SIMPLE_METHOD (method);
+  GstDeinterlaceMethodClass *dm_class = GST_DEINTERLACE_METHOD_GET_CLASS (self);
+  guint8 *out;
+  const guint8 *field0, *field1, *field2, *fieldp;
+  guint cur_field_flags = history[cur_field_idx].flags;
+  gint i, offset;
+
+  g_assert (self->interpolate_scanline_packed != NULL);
+  g_assert (self->copy_scanline_packed != NULL);
+
+  for (i = 0; i < 2; i++) {
+    offset = self->parent.offset[i];
+
+    out = GST_BUFFER_DATA (outbuf) + offset;
+
+    fieldp = NULL;
+    if (cur_field_idx > 0) {
+      fieldp = GST_BUFFER_DATA (history[cur_field_idx - 1].buf) + offset;
+    }
+
+    field0 = GST_BUFFER_DATA (history[cur_field_idx].buf) + offset;
+
+    g_assert (dm_class->fields_required <= 4);
+
+    field1 = NULL;
+    if (cur_field_idx + 1 < history_count) {
+      field1 = GST_BUFFER_DATA (history[cur_field_idx + 1].buf) + offset;
+    }
+
+    field2 = NULL;
+    if (cur_field_idx + 2 < history_count) {
+      field2 = GST_BUFFER_DATA (history[cur_field_idx + 2].buf) + offset;
     }
 
     gst_deinterlace_simple_method_deinterlace_frame_planar_plane (self, out,
-        field0, field1, field2, field3, cur_field_flags, i, copy_scanline,
-        interpolate_scanline);
+        field0, field1, field2, fieldp, cur_field_flags, i,
+        self->copy_scanline_packed, self->interpolate_scanline_packed);
   }
 }
 
@@ -615,6 +674,14 @@ gst_deinterlace_simple_method_setup (GstDeinterlaceMethod * method,
       self->interpolate_scanline_packed = klass->interpolate_scanline_bgr;
       self->copy_scanline_packed = klass->copy_scanline_bgr;
       break;
+    case GST_VIDEO_FORMAT_NV12:
+      self->interpolate_scanline_packed = klass->interpolate_scanline_nv12;
+      self->copy_scanline_packed = klass->copy_scanline_nv12;
+      break;
+    case GST_VIDEO_FORMAT_NV21:
+      self->interpolate_scanline_packed = klass->interpolate_scanline_nv21;
+      self->copy_scanline_packed = klass->copy_scanline_nv21;
+      break;
     case GST_VIDEO_FORMAT_I420:
     case GST_VIDEO_FORMAT_YV12:
     case GST_VIDEO_FORMAT_Y444:
@@ -671,6 +738,10 @@ gst_deinterlace_simple_method_class_init (GstDeinterlaceSimpleMethodClass
       gst_deinterlace_simple_method_deinterlace_frame_planar;
   dm_class->deinterlace_frame_y41b =
       gst_deinterlace_simple_method_deinterlace_frame_planar;
+  dm_class->deinterlace_frame_nv12 =
+      gst_deinterlace_simple_method_deinterlace_frame_nv12;
+  dm_class->deinterlace_frame_nv21 =
+      gst_deinterlace_simple_method_deinterlace_frame_nv12;
   dm_class->fields_required = 2;
   dm_class->setup = gst_deinterlace_simple_method_setup;
   dm_class->supported = gst_deinterlace_simple_method_supported;
@@ -691,6 +762,10 @@ gst_deinterlace_simple_method_class_init (GstDeinterlaceSimpleMethodClass
       gst_deinterlace_simple_method_interpolate_scanline_packed;
   klass->copy_scanline_uyvy =
       gst_deinterlace_simple_method_copy_scanline_packed;
+  klass->interpolate_scanline_nv12 =
+      gst_deinterlace_simple_method_interpolate_scanline_packed;
+  klass->copy_scanline_nv12 =
+      gst_deinterlace_simple_method_copy_scanline_packed;
 
   klass->interpolate_scanline_argb =
       gst_deinterlace_simple_method_interpolate_scanline_packed;
@@ -709,7 +784,6 @@ gst_deinterlace_simple_method_class_init (GstDeinterlaceSimpleMethodClass
       gst_deinterlace_simple_method_interpolate_scanline_packed;
   klass->copy_scanline_bgra =
       gst_deinterlace_simple_method_copy_scanline_packed;
-
   klass->interpolate_scanline_rgb =
       gst_deinterlace_simple_method_interpolate_scanline_packed;
   klass->copy_scanline_rgb = gst_deinterlace_simple_method_copy_scanline_packed;
