@@ -95,6 +95,8 @@ struct _GstFlvDemux
   gboolean audio_linked;
   GstBuffer * audio_codec_data;
   GstClockTime audio_start;
+  guint32 last_audio_pts;
+  GstClockTime audio_time_offset;
 
   /* Video infos */
   guint32 w;
@@ -109,6 +111,9 @@ struct _GstFlvDemux
   gboolean got_par;
   GstBuffer * video_codec_data;
   GstClockTime video_start;
+  guint32 last_video_pts;
+  GstClockTime video_time_offset;
+  gdouble framerate;
 
   gboolean random_access;
   gboolean need_header;
@@ -123,6 +128,7 @@ struct _GstFlvDemux
   gboolean seeking;
   gboolean building_index;
   gboolean indexed; /* TRUE if index is completely built */
+  gboolean upstream_seekable; /* TRUE if upstream is seekable */
   gint64 file_size;
   GstEvent *seek_event;
   gint64 seek_time;
