@@ -190,10 +190,10 @@ gst_deinterleave_base_init (gpointer g_class)
       "Iain <iain@prettypeople.org>, "
       "Sebastian Dröge <slomo@circular-chaos.org>");
 
-  gst_element_class_add_pad_template (gstelement_class,
-      gst_static_pad_template_get (&sink_template));
-  gst_element_class_add_pad_template (gstelement_class,
-      gst_static_pad_template_get (&src_template));
+  gst_element_class_add_static_pad_template (gstelement_class,
+      &sink_template);
+  gst_element_class_add_static_pad_template (gstelement_class,
+      &src_template);
 }
 
 static void
