@@ -1,5 +1,6 @@
 /* GStreamer Matroska muxer/demuxer
  * (c) 2003 Ronald Bultje <rbultje@ronald.bitfreak.net>
+ * (c) 2011 Debarshi Ray <rishi@gnu.org>
  *
  * matroska-demux.h: matroska file/stream demuxer definition
  *
@@ -23,10 +24,10 @@
 #define __GST_MATROSKA_DEMUX_H__
 
 #include <gst/gst.h>
-#include <gst/base/gstadapter.h>
 
 #include "ebml-read.h"
 #include "matroska-ids.h"
+#include "matroska-read-common.h"
 
 G_BEGIN_DECLS
 
@@ -41,80 +42,66 @@ G_BEGIN_DECLS
 #define GST_IS_MATROSKA_DEMUX_CLASS(klass) \
   (G_TYPE_CHECK_CLASS_TYPE ((klass), GST_TYPE_MATROSKA_DEMUX))
 
-typedef enum {
-  GST_MATROSKA_DEMUX_STATE_START,
-  GST_MATROSKA_DEMUX_STATE_HEADER,
-  GST_MATROSKA_DEMUX_STATE_DATA
-} GstMatroskaDemuxState;
-
 typedef struct _GstMatroskaDemux {
-  GstEbmlRead              parent;
+  GstElement              parent;
 
   /* < private > */
 
-  GstIndex                *element_index;
-  gint                     element_index_writer_id;
+  GstMatroskaReadCommon    common;
 
   /* pads */
-  GstPad                  *sinkpad;
-  GPtrArray               *src;
   GstClock                *clock;
-  guint                    num_streams;
   guint                    num_v_streams;
   guint                    num_a_streams;
   guint                    num_t_streams;
 
-  /* metadata */
-  gchar                   *muxing_app;
-  gchar                   *writing_app;
-  gint64                   created;
-
   /* state */
-  GstMatroskaDemuxState    state;
+  gboolean                 streaming;
   guint                    level_up;
   guint64                  seek_block;
+  gboolean                 seek_first;
 
   /* did we parse cues/tracks/segmentinfo already? */
-  gboolean                 index_parsed;
   gboolean                 tracks_parsed;
-  gboolean                 segmentinfo_parsed;
-  gboolean                 attachments_parsed;
-  GList                   *tags_parsed;
+  GList                   *seek_parsed;
 
-  /* start-of-segment */
-  guint64                  ebml_segment_start;
-
-  /* a cue (index) table */
-  GArray                  *index;
-
-  /* timescale in the file */
-  guint64                  time_scale;
+  /* cluster positions (optional) */
+  GArray                  *clusters;
 
   /* keeping track of playback position */
-  GstSegment               segment;
   gboolean                 segment_running;
   GstClockTime             last_stop_end;
+  GstClockTime             stream_start_time;
 
   GstEvent                *close_segment;
   GstEvent                *new_segment;
-  GstTagList              *global_tags;
 
-  /* push based mode usual suspects */
-  guint64                  offset;
-  GstAdapter              *adapter;
   /* some state saving */
   GstClockTime             cluster_time;
   guint64                  cluster_offset;
+  guint64                  first_cluster_offset;
+  guint64                  next_cluster_offset;
+
+  /* index stuff */
+  gboolean                 seekable;
+  gboolean                 building_index;
+  guint64                  index_offset;
+  GstEvent                *seek_event;
+  gboolean                 need_newsegment;
 
   /* reverse playback */
   GArray                  *seek_index;
   gint                     seek_entry;
-  gint64                   from_offset;
-  gint64                   to_offset;
+
+  /* gap handling */
+  guint64                  max_gap_time;
+
+  /* for non-finalized files, with invalid segment duration */
+  gboolean                 invalid_duration;
 } GstMatroskaDemux;
 
 typedef struct _GstMatroskaDemuxClass {
-  GstEbmlReadClass parent;
+  GstElementClass parent;
 } GstMatroskaDemuxClass;
 
 gboolean gst_matroska_demux_plugin_init (GstPlugin *plugin);
