@@ -23,18 +23,23 @@
 
 #include "gstrtpdepay.h"
 #include "gstrtpac3depay.h"
+#include "gstrtpac3pay.h"
 #include "gstrtpbvdepay.h"
 #include "gstrtpbvpay.h"
 #include "gstrtpceltdepay.h"
 #include "gstrtpceltpay.h"
 #include "gstrtpdvdepay.h"
 #include "gstrtpdvpay.h"
+#include "gstrtpgstdepay.h"
+#include "gstrtpgstpay.h"
 #include "gstrtpilbcdepay.h"
 #include "gstrtpilbcpay.h"
 #include "gstrtppcmupay.h"
 #include "gstrtppcmapay.h"
 #include "gstrtppcmadepay.h"
 #include "gstrtppcmudepay.h"
+#include "gstrtpg722depay.h"
+#include "gstrtpg722pay.h"
 #include "gstrtpg723depay.h"
 #include "gstrtpg723pay.h"
 #include "gstrtpg726depay.h"
@@ -47,6 +52,7 @@
 #include "gstrtpamrdepay.h"
 #include "gstrtpmpapay.h"
 #include "gstrtpmpadepay.h"
+#include "gstrtpmparobustdepay.h"
 #include "gstrtpmpvdepay.h"
 #include "gstrtpmpvpay.h"
 #include "gstrtph263pdepay.h"
@@ -71,6 +77,7 @@
 #include "gstrtpmp4apay.h"
 #include "gstrtpmp4gdepay.h"
 #include "gstrtpmp4gpay.h"
+#include "gstrtpqcelpdepay.h"
 #include "gstrtpqdmdepay.h"
 #include "gstrtpsirenpay.h"
 #include "gstrtpsirendepay.h"
@@ -93,6 +100,9 @@ plugin_init (GstPlugin * plugin)
   if (!gst_rtp_ac3_depay_plugin_init (plugin))
     return FALSE;
 
+  if (!gst_rtp_ac3_pay_plugin_init (plugin))
+    return FALSE;
+
   if (!gst_rtp_bv_depay_plugin_init (plugin))
     return FALSE;
 
@@ -111,12 +121,24 @@ plugin_init (GstPlugin * plugin)
   if (!gst_rtp_dv_pay_plugin_init (plugin))
     return FALSE;
 
+  if (!gst_rtp_gst_depay_plugin_init (plugin))
+    return FALSE;
+
+  if (!gst_rtp_gst_pay_plugin_init (plugin))
+    return FALSE;
+
   if (!gst_rtp_ilbc_pay_plugin_init (plugin))
     return FALSE;
 
   if (!gst_rtp_ilbc_depay_plugin_init (plugin))
     return FALSE;
 
+  if (!gst_rtp_g722_depay_plugin_init (plugin))
+    return FALSE;
+
+  if (!gst_rtp_g722_pay_plugin_init (plugin))
+    return FALSE;
+
   if (!gst_rtp_g723_depay_plugin_init (plugin))
     return FALSE;
 
@@ -165,6 +187,9 @@ plugin_init (GstPlugin * plugin)
   if (!gst_rtp_mpa_pay_plugin_init (plugin))
     return FALSE;
 
+  if (!gst_rtp_mpa_robust_depay_plugin_init (plugin))
+    return FALSE;
+
   if (!gst_rtp_mpv_depay_plugin_init (plugin))
     return FALSE;
 
@@ -237,6 +262,9 @@ plugin_init (GstPlugin * plugin)
   if (!gst_rtp_mp4g_pay_plugin_init (plugin))
     return FALSE;
 
+  if (!gst_rtp_qcelp_depay_plugin_init (plugin))
+    return FALSE;
+
   if (!gst_rtp_qdm2_depay_plugin_init (plugin))
     return FALSE;
 
