@@ -80,10 +80,10 @@ gst_rtp_L16_pay_base_init (gpointer klass)
 {
   GstElementClass *element_class = GST_ELEMENT_CLASS (klass);
 
-  gst_element_class_add_pad_template (element_class,
-      gst_static_pad_template_get (&gst_rtp_L16_pay_src_template));
-  gst_element_class_add_pad_template (element_class,
-      gst_static_pad_template_get (&gst_rtp_L16_pay_sink_template));
+  gst_element_class_add_static_pad_template (element_class,
+      &gst_rtp_L16_pay_src_template);
+  gst_element_class_add_static_pad_template (element_class,
+      &gst_rtp_L16_pay_sink_template);
 
   gst_element_class_set_details_simple (element_class, "RTP audio payloader",
       "Codec/Payloader/Network/RTP",
