@@ -66,12 +66,12 @@ gst_rtp_bv_pay_base_init (gpointer klass)
 {
   GstElementClass *element_class = GST_ELEMENT_CLASS (klass);
 
-  gst_element_class_add_pad_template (element_class,
-      gst_static_pad_template_get (&gst_rtp_bv_pay_sink_template));
-  gst_element_class_add_pad_template (element_class,
-      gst_static_pad_template_get (&gst_rtp_bv_pay_src_template));
+  gst_element_class_add_static_pad_template (element_class,
+      &gst_rtp_bv_pay_sink_template);
+  gst_element_class_add_static_pad_template (element_class,
+      &gst_rtp_bv_pay_src_template);
   gst_element_class_set_details_simple (element_class, "RTP BV Payloader",
-      "Codec/Payloader/Network",
+      "Codec/Payloader/Network/RTP",
       "Packetize BroadcomVoice audio streams into RTP packets (RFC 4298)",
       "Wim Taymans <wim.taymans@collabora.co.uk>");
 }
@@ -93,10 +93,8 @@ gst_rtp_bv_pay_class_init (GstRTPBVPayClass * klass)
 static void
 gst_rtp_bv_pay_init (GstRTPBVPay * rtpbvpay, GstRTPBVPayClass * klass)
 {
-  GstBaseRTPPayload *basertppayload;
   GstBaseRTPAudioPayload *basertpaudiopayload;
 
-  basertppayload = GST_BASE_RTP_PAYLOAD (rtpbvpay);
   basertpaudiopayload = GST_BASE_RTP_AUDIO_PAYLOAD (rtpbvpay);
 
   rtpbvpay->mode = -1;
@@ -219,5 +217,5 @@ gboolean
 gst_rtp_bv_pay_plugin_init (GstPlugin * plugin)
 {
   return gst_element_register (plugin, "rtpbvpay",
-      GST_RANK_NONE, GST_TYPE_RTP_BV_PAY);
+      GST_RANK_SECONDARY, GST_TYPE_RTP_BV_PAY);
 }
