@@ -70,12 +70,12 @@ gst_rtp_celt_pay_base_init (gpointer klass)
 {
   GstElementClass *element_class = GST_ELEMENT_CLASS (klass);
 
-  gst_element_class_add_pad_template (element_class,
-      gst_static_pad_template_get (&gst_rtp_celt_pay_sink_template));
-  gst_element_class_add_pad_template (element_class,
-      gst_static_pad_template_get (&gst_rtp_celt_pay_src_template));
+  gst_element_class_add_static_pad_template (element_class,
+      &gst_rtp_celt_pay_sink_template);
+  gst_element_class_add_static_pad_template (element_class,
+      &gst_rtp_celt_pay_src_template);
   gst_element_class_set_details_simple (element_class, "RTP CELT payloader",
-      "Codec/Payloader/Network",
+      "Codec/Payloader/Network/RTP",
       "Payload-encodes CELT audio into a RTP packet",
       "Wim Taymans <wim.taymans@gmail.com>");
 
@@ -141,7 +141,16 @@ gst_rtp_celt_pay_add_queued (GstRtpCELTPay * rtpceltpay, GstBuffer * buffer,
   g_queue_push_tail (rtpceltpay->queue, buffer);
   rtpceltpay->sbytes += ssize;
   rtpceltpay->bytes += size;
-  rtpceltpay->qduration += duration;
+  /* only add durations when we have a valid previous duration */
+  if (rtpceltpay->qduration != -1) {
+    if (duration != -1)
+      /* only add valid durations */
+      rtpceltpay->qduration += duration;
+    else
+      /* if we add a buffer without valid duration, our total queued duration
+       * becomes unknown */
+      rtpceltpay->qduration = -1;
+  }
 }
 
 static gboolean
@@ -177,9 +186,10 @@ gst_rtp_celt_pay_getcaps (GstBaseRTPPayload * payload, GstPad * pad)
       if (frame_size)
         gst_structure_set (s, "frame-size", G_TYPE_INT, frame_size, NULL);
 
-      if ((params = gst_structure_get_string (ps, "encoding-params")))
+      if ((params = gst_structure_get_string (ps, "encoding-params"))) {
         channels = atoi (params);
-      gst_structure_fixate_field_nearest_int (s, "channels", channels);
+        gst_structure_fixate_field_nearest_int (s, "channels", channels);
+      }
 
       GST_DEBUG_OBJECT (payload, "clock-rate=%d frame-size=%d channels=%d",
           clock_rate, frame_size, channels);
@@ -396,9 +406,12 @@ gst_rtp_celt_pay_handle_buffer (GstBaseRTPPayload * basepayload,
 
   GST_DEBUG_OBJECT (rtpceltpay, "bytes for size %u", ssize);
 
-  /* calculate the size and duration of the packet */
+  /* calculate what the new size and duration would be of the packet */
   payload_len = ssize + size + rtpceltpay->bytes + rtpceltpay->sbytes;
-  packet_dur = rtpceltpay->qduration + duration;
+  if (rtpceltpay->qduration != -1 && duration != -1)
+    packet_dur = rtpceltpay->qduration + duration;
+  else
+    packet_dur = 0;
 
   packet_len = gst_rtp_buffer_calc_packet_len (payload_len, 0, 0);
 
@@ -460,5 +473,5 @@ gboolean
 gst_rtp_celt_pay_plugin_init (GstPlugin * plugin)
 {
   return gst_element_register (plugin, "rtpceltpay",
-      GST_RANK_NONE, GST_TYPE_RTP_CELT_PAY);
+      GST_RANK_SECONDARY, GST_TYPE_RTP_CELT_PAY);
 }
