@@ -63,14 +63,14 @@ gst_rtp_depay_base_init (gpointer klass)
 {
   GstElementClass *gstelement_class = GST_ELEMENT_CLASS (klass);
 
-  gst_element_class_add_pad_template (gstelement_class,
-      gst_static_pad_template_get (&gst_rtp_depay_src_rtp_template));
-  gst_element_class_add_pad_template (gstelement_class,
-      gst_static_pad_template_get (&gst_rtp_depay_src_rtcp_template));
-  gst_element_class_add_pad_template (gstelement_class,
-      gst_static_pad_template_get (&gst_rtp_depay_sink_rtp_template));
-  gst_element_class_add_pad_template (gstelement_class,
-      gst_static_pad_template_get (&gst_rtp_depay_sink_rtcp_template));
+  gst_element_class_add_static_pad_template (gstelement_class,
+      &gst_rtp_depay_src_rtp_template);
+  gst_element_class_add_static_pad_template (gstelement_class,
+      &gst_rtp_depay_src_rtcp_template);
+  gst_element_class_add_static_pad_template (gstelement_class,
+      &gst_rtp_depay_sink_rtp_template);
+  gst_element_class_add_static_pad_template (gstelement_class,
+      &gst_rtp_depay_sink_rtcp_template);
   gst_element_class_set_details_simple (gstelement_class,
       "Dummy RTP session manager", "Codec/Depayloader/Network/RTP",
       "Accepts raw RTP and RTCP packets and sends them forward",
