@@ -96,10 +96,10 @@ gst_rtp_ilbc_depay_base_init (gpointer klass)
 {
   GstElementClass *element_class = GST_ELEMENT_CLASS (klass);
 
-  gst_element_class_add_pad_template (element_class,
-      gst_static_pad_template_get (&gst_rtp_ilbc_depay_src_template));
-  gst_element_class_add_pad_template (element_class,
-      gst_static_pad_template_get (&gst_rtp_ilbc_depay_sink_template));
+  gst_element_class_add_static_pad_template (element_class,
+      &gst_rtp_ilbc_depay_src_template);
+  gst_element_class_add_static_pad_template (element_class,
+      &gst_rtp_ilbc_depay_sink_template);
   gst_element_class_set_details_simple (element_class, "RTP iLBC depayloader",
       "Codec/Depayloader/Network/RTP",
       "Extracts iLBC audio from RTP packets (RFC 3952)",
