@@ -91,10 +91,10 @@ GST_BOILERPLATE (GstRtpMP4GPay, gst_rtp_mp4g_pay, GstBaseRTPPayload,
 {
   GstElementClass *element_class = GST_ELEMENT_CLASS (klass);
 
-  gst_element_class_add_pad_template (element_class,
-      gst_static_pad_template_get (&gst_rtp_mp4g_pay_src_template));
-  gst_element_class_add_pad_template (element_class,
-      gst_static_pad_template_get (&gst_rtp_mp4g_pay_sink_template));
+  gst_element_class_add_static_pad_template (element_class,
+      &gst_rtp_mp4g_pay_src_template);
+  gst_element_class_add_static_pad_template (element_class,
+      &gst_rtp_mp4g_pay_sink_template);
 
   gst_element_class_set_details_simple (element_class, "RTP MPEG4 ES payloader",
       "Codec/Payloader/Network/RTP",
@@ -451,11 +451,8 @@ gst_rtp_mp4g_pay_flush (GstRtpMP4GPay * rtpmp4gpay)
   guint avail, total;
   GstBuffer *outbuf;
   GstFlowReturn ret;
-  gboolean fragmented;
   guint mtu;
 
-  fragmented = FALSE;
-
   /* the data available in the adapter is either smaller
    * than the MTU or bigger. In the case it is smaller, the complete
    * adapter contents can be put in one packet. In the case the
@@ -544,7 +541,6 @@ gst_rtp_mp4g_pay_flush (GstRtpMP4GPay * rtpmp4gpay)
     ret = gst_basertppayload_push (GST_BASE_RTP_PAYLOAD (rtpmp4gpay), outbuf);
 
     avail -= payload_len;
-    fragmented = TRUE;
   }
 
   return ret;
