@@ -41,9 +41,13 @@
 #include <errno.h>
 #include <string.h>
 
+#include "gst/glib-compat-private.h"
+
 GST_DEBUG_CATEGORY_STATIC (multiudpsink_debug);
 #define GST_CAT_DEFAULT (multiudpsink_debug)
 
+#define UDP_MAX_SIZE 65507
+
 static GstStaticPadTemplate sink_template = GST_STATIC_PAD_TEMPLATE ("sink",
     GST_PAD_SINK,
     GST_PAD_ALWAYS,
@@ -170,8 +174,7 @@ gst_multiudpsink_base_init (gpointer g_class)
 {
   GstElementClass *element_class = GST_ELEMENT_CLASS (g_class);
 
-  gst_element_class_add_pad_template (element_class,
-      gst_static_pad_template_get (&sink_template));
+  gst_element_class_add_static_pad_template (element_class, &sink_template);
 
   gst_element_class_set_details_simple (element_class, "UDP packet sender",
       "Sink/Network",
@@ -212,7 +215,8 @@ gst_multiudpsink_class_init (GstMultiUDPSinkClass * klass)
    * of destinations.
    */
   gst_multiudpsink_signals[SIGNAL_ADD] =
-      g_signal_new ("add", G_TYPE_FROM_CLASS (klass), G_SIGNAL_RUN_LAST,
+      g_signal_new ("add", G_TYPE_FROM_CLASS (klass),
+      G_SIGNAL_RUN_LAST | G_SIGNAL_ACTION,
       G_STRUCT_OFFSET (GstMultiUDPSinkClass, add),
       NULL, NULL, gst_udp_marshal_VOID__STRING_INT, G_TYPE_NONE, 2,
       G_TYPE_STRING, G_TYPE_INT);
@@ -226,7 +230,8 @@ gst_multiudpsink_class_init (GstMultiUDPSinkClass * klass)
    * clients.
    */
   gst_multiudpsink_signals[SIGNAL_REMOVE] =
-      g_signal_new ("remove", G_TYPE_FROM_CLASS (klass), G_SIGNAL_RUN_LAST,
+      g_signal_new ("remove", G_TYPE_FROM_CLASS (klass),
+      G_SIGNAL_RUN_LAST | G_SIGNAL_ACTION,
       G_STRUCT_OFFSET (GstMultiUDPSinkClass, remove),
       NULL, NULL, gst_udp_marshal_VOID__STRING_INT, G_TYPE_NONE, 2,
       G_TYPE_STRING, G_TYPE_INT);
@@ -237,7 +242,8 @@ gst_multiudpsink_class_init (GstMultiUDPSinkClass * klass)
    * Clear the list of clients.
    */
   gst_multiudpsink_signals[SIGNAL_CLEAR] =
-      g_signal_new ("clear", G_TYPE_FROM_CLASS (klass), G_SIGNAL_RUN_LAST,
+      g_signal_new ("clear", G_TYPE_FROM_CLASS (klass),
+      G_SIGNAL_RUN_LAST | G_SIGNAL_ACTION,
       G_STRUCT_OFFSET (GstMultiUDPSinkClass, clear),
       NULL, NULL, g_cclosure_marshal_VOID__VOID, G_TYPE_NONE, 0);
   /**
@@ -252,7 +258,8 @@ gst_multiudpsink_class_init (GstMultiUDPSinkClass * klass)
    *           connect_time (in epoch seconds), disconnect_time (in epoch seconds)
    */
   gst_multiudpsink_signals[SIGNAL_GET_STATS] =
-      g_signal_new ("get-stats", G_TYPE_FROM_CLASS (klass), G_SIGNAL_RUN_LAST,
+      g_signal_new ("get-stats", G_TYPE_FROM_CLASS (klass),
+      G_SIGNAL_RUN_LAST | G_SIGNAL_ACTION,
       G_STRUCT_OFFSET (GstMultiUDPSinkClass, get_stats),
       NULL, NULL, gst_udp_marshal_BOXED__STRING_INT, G_TYPE_VALUE_ARRAY, 2,
       G_TYPE_STRING, G_TYPE_INT);
@@ -291,7 +298,7 @@ gst_multiudpsink_class_init (GstMultiUDPSinkClass * klass)
           G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
   g_object_class_install_property (G_OBJECT_CLASS (klass), PROP_BYTES_SERVED,
       g_param_spec_uint64 ("bytes-served", "Bytes served",
-          "Total number of bytes send to all clients", 0, G_MAXUINT64, 0,
+          "Total number of bytes sent to all clients", 0, G_MAXUINT64, 0,
           G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
   g_object_class_install_property (gobject_class, PROP_SOCKFD,
       g_param_spec_int ("sockfd", "Socket Handle",
@@ -496,6 +503,11 @@ gst_multiudpsink_render (GstBaseSink * bsink, GstBuffer * buffer)
   size = GST_BUFFER_SIZE (buffer);
   data = GST_BUFFER_DATA (buffer);
 
+  if (size > UDP_MAX_SIZE) {
+    GST_WARNING ("Attempting to send a UDP packet larger than maximum "
+        "size (%d > %d)", size, UDP_MAX_SIZE);
+  }
+
   sink->bytes_to_serve += size;
 
   /* grab lock while iterating and sending to clients, this should be
@@ -586,6 +598,11 @@ gst_multiudpsink_render_list (GstBaseSink * bsink, GstBufferList * list)
     msg.msg_iov = iov;
 
     while ((buf = gst_buffer_list_iterator_next (it))) {
+      if (GST_BUFFER_SIZE (buf) > UDP_MAX_SIZE) {
+        GST_WARNING ("Attempting to send a UDP packet larger than maximum "
+            "size (%d > %d)", GST_BUFFER_SIZE (buf), UDP_MAX_SIZE);
+      }
+
       msg.msg_iov[msg.msg_iovlen].iov_len = GST_BUFFER_SIZE (buf);
       msg.msg_iov[msg.msg_iovlen].iov_base = GST_BUFFER_DATA (buf);
       msg.msg_iovlen++;
@@ -913,11 +930,7 @@ gst_multiudpsink_init_send (GstMultiUDPSink * sink)
   GList *clients;
   GstUDPClient *client;
   int sndsize, ret;
-#ifdef G_OS_WIN32
-  gint len;
-#else
-  guint len;
-#endif
+  socklen_t len;
 
   if (sink->sockfd == -1) {
     GST_DEBUG_OBJECT (sink, "creating sockets");
